extern "C" void set_sigma_(int& CDR);
extern "C" double wsigma_(int& ZI, int& AI, int& ZF,int& AF, double& ENERGY, double& FH);
extern "C" double sighe_(int& ZI, int& ZF, double& ENERGY);
extern "C" void sectotal_(int& IT, double ER[], double* SIGTOTA);
//extern "C" void sectotal_(int& IT, double& ER, double& SIGTOTAL);
#include <stdio.h>
#include <stdlib.h>
#include <TROOT.h>
#include <TGraph.h>
#include <TFile.h>
#include <TF1.h>

double totccxsec(double* x, double* par)//total charge changing cross-section parametrization at 1.5 GeV/n
{
  double xsH  = par[0] * pow(x[0], par[1]);//H target
  double xsHe = par[2] * pow(x[0], par[3]);//He target
  double xs = 0;//cross-section in mbarn
  if(par[4]>0 && par[4]<1) xs = par[4]*xsH + (1-par[4])*xsHe;//ISM target par[4] is % of H
  else if(par[4]==0.0) xs = xsHe;
  else xs = xsH;
  return xs;
}

int main(int argc, char* argv[])
{
  //Input options
  if( (argc<=1 || argc>3)){
    printf("Usage ./xsec argv[0] argv[1]\n");
    printf("argv[0] fraction of H in the target: 0<=fH<=1\n");
    printf("argv[1] values: 0 (compute xs in mbarn), 1 (compute xs fraction)\n");
    return 0;
  }
  int NUMOPT = atoi(argv[2]);
  double fH = atof(argv[1]);//% of H in the target (fH=1.0 H; fH=0 He; 0<fH<1  ISM with fH % of H)
  if(fH<0 || fH>1){
    printf("argv[0] value is not valid. fH (H abundance in target) must be 0<=fH<=1\n");
    return 0;
  }
  if(NUMOPT!=0 && NUMOPT!=1){
    printf("argv[1] value is not valid. Options: 0 (compute xs in mbarn), 1 (compute xs fraction)\n");
    return 0;
  }
  
  //Save partial cross section in ROOT TGraphs
  TFile* rfile = new TFile("xsec.root","RECREATE");
  TGraph* gFe[26], *gSi[14], *gO[8], *gC[6], *gB[5];
  for(int i=0; i<26; i++){
    gFe[i] = new TGraph();
    gFe[i]->SetName(Form("Fe%d",i));
  }
  for(int i=0; i<14; i++){
    gSi[i] = new TGraph();
    gSi[i]->SetName(Form("Si%d",i));
  }
  for(int i=0; i<8; i++){
    gO[i] = new TGraph();
    gO[i]->SetName(Form("O%d",i));
  }
  for(int i=0; i<6; i++){
    gC[i] = new TGraph();
    gC[i]->SetName(Form("C%d",i));
  }
  for(int i=0; i<5; i++){
    gB[i] = new TGraph();
    gB[i]->SetName(Form("B%d",i));
  }

  //atomic mass in amu
  double mA[30]={0, 1.00794, 4.002602, 6.941,  9.012, 10.811, 12.0107, 14.007, 15.9994,  18.9994, 20.1797, 22.98977, 24.3050, 26.981538, 28.0855, 
		 30.973761, 32.065, 35.453, 39.948, 39.0983,  40.078, 44.955910, 47.867, 50.9415, 51.9961, 54.938049, 55.845, 58.933, 58.6934};//masse in amu

  //*************************************************************************************
  //total charge changing cross section (in mbarn) at 1.5 GeV/n mbarn
  //parameterisation from fit  Ap + target (table IV,VI e fig.7, Webber, Phys Rev C41 (1990), 520)
  //*************************************************************************************
  TF1* ftccxs = new TF1("ftccxs",totccxsec,5,65,5);
  //TF1* ftccxs = new TF1("ftccxs","[0]*pow(x,[1])",5,65,5);
  //params:             xsH=p0*Ap^p1   xsHe=p2*Ap^p3  p4 is H% in ISM
  ftccxs->SetParameters( 34.9, 0.75,     88.7, 0.64,   fH         );
  ftccxs->SetLineColor(4);

  //*************************************************************************************
  //compute individual cross sections i.e. for incident beam (ZI, AI, ENERGY)
  //interacting in a H (or He) target and producing  a fragment (ZF, AF)
  //*************************************************************************************
  int dev = 5;
  set_sigma_(dev);//initialize parameters of Webber Cross section program WNEWTR.f
  int verbose = 0;
  double XS[30][30] = {{0},{0}};//first index: Zi; second index: Zf

  double amu = 1.66053886e-27*1e3;//amu in g
  
  for(int s=-1; s<500; s++){//energy scan
    double E = 100+s*10;//from 100 MeV/n to 5000 MeV/n
    if(s==-1) E=600;
    if(E==600) verbose = 1;
    else verbose = 0;
    
    int STARTZ=3;//start from Lithium
    int N=28;//elements up to Nickel
    if(verbose==1){
      printf("*********************************************************\n");
      if(fH==1.0) printf("cross-sections Zi + H --> Zf in mbarn at E=%.1f MeV/n\n",E);
      else if(fH==0.0) printf("cross-sections Zi + He --> Zf in mbarn at E=%.1f MeV/n\n",E);
      else printf("cross-sections Zi + ISM --> Zf in mbarn at E=%.1f MeV/n\n",E);
      printf("*********************************************************\n");
      printf("Zf");
      for(int zf=STARTZ; zf<N; zf++)
	printf("|%4d ", zf);
      printf("|xs sum|tcxs1.5|L(g/cm2)");
      printf("\n");
      printf("Zi --------------------------------------------------------------------------------------------------------------------------------------------------------------------------\n");
    }
    
    for(int zi=STARTZ; zi<N+1; zi++){//LOOP over zi of projectile
      int Ai = (int) (mA[zi]+0.5); 
      if(verbose==1) printf("%2d",zi);
      float totccxs = ftccxs->Eval(Ai);//total cc cross section at 1.5 GeV/n
      double MFP = ( fH*mA[1] + (1.0-fH)*mA[2])/ (totccxs*1e-27) *amu;//mean free path at 1.5 GeV/n (*1e-27 mbarn -> cm2)

      float sum_xs = 0;//sum of individual cross sections for each projectile zi
      for(int zf=STARTZ; zf<N; zf++){//LOOP over zf of the fragment
	if(zf>=STARTZ && zf<zi){
	  int Af = (int) (mA[zf]+0.5);//mass number of the most abundant isotope
	  double xs = 0;
	  if(zi >=0){
	    for(int k=-20; k<20; k++){//scan the isotopes of zf. Use Af as center and go from Af-20 to Af+20; 
	      int af = Af + k;
	      if(af>0)
		xs += (wsigma_(zi, Ai, zf, af, E, fH));//sum xs of the isotopes of the same zf fragment
	    }
	  }

	  if(E==600) XS[zi][zf] = xs;
	  sum_xs += xs;
	  
	  if(NUMOPT==1){//compute fraction of xs to the total cc cross-section
	    xs = xs/totccxs;
	    if(verbose==1) printf("|%5.3f",xs);
	  }
	  else{
	    if(verbose==1) printf("|%5.1f",xs);
	  }
	  
	  //save xs vs. energy for B,C,O, Si, Fe to compare with figures 4-7 of  Webber et al. ApJ S144 (2003) 153)
	  if(s>=0){
	    double norm = 1.0;
	    //if(XS[zi][zf]>0) norm = XS[zi][zf];//uncomment to normalize xs to the value at 600 MeV
	    if(zi==26){
	      if(zf<26 && zf>=0) gFe[zf]->SetPoint(s, E, xs/norm);
	    }
	    if(zi==14){
	      if(zf<14 && zf>=0) gSi[zf]->SetPoint(s, E, xs/norm);
	    }
	    if(zi==8){
	      if(zf<8 && zf>=0) gO[zf]->SetPoint(s, E, xs/norm);
	    }
	    if(zi==6){
	      if(zf<6 && zf>=0) gC[zf]->SetPoint(s, E, xs/norm);
	    }
	    if(zi==5){
	      if(zf<5 && zf>=0) gB[zf]->SetPoint(s, E, xs/norm);
	    }
	  }
	}
	else{
	  if(verbose==1) printf("|     ");
	}
      }//end LOOP zf
      if(verbose==1){
	printf("|%6.1f |%6.1f |%6.3f", sum_xs, totccxs, MFP);
	printf("\n");
      }
    }//end LOOP zi
  }//end energy scan
  
  //save graphs to ROOT file
  for(int i=0; i<26; i++){
    gFe[i]->Write();
    if(i<14) gSi[i]->Write();
    if(i<8)  gO[i]->Write(); 
    if(i<6)  gC[i]->Write();
    if(i<5)  gB[i]->Write();
  }
  rfile->Close();

  //double ER[10] = {300, 200,1500,200,200, 200, 200,200,200,200};
  //double SIGTOTAL[10];
  //int IT = 12;
  //sectotal_(IT, ER, SIGTOTAL);
  return 0;
}
